#ifndef __BS_H__
#define __BS_H__


#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

/*
 * Tuple description
 */
enum {
    FIELD_ALBUM = 0,
    FIELD_TITLE,
    FIELD_LENGTH,
    FIELD_AUTHOR,
    FIELD_PERFORMED_BY,
    FIELD_YEAR,

    NUM_FIELDS,
    FIELD_NONE = -1
};

#define MAX_FLEN                40
#define FLEN_ALBUM              40
#define FLEN_TITLE              40
#define FLEN_LENGTH             6
#define FLEN_AUTHOR             30
#define FLEN_PERFORMED_BY       30
#define FLEN_YEAR               4

typedef struct _FieldDsc {
    int   len;
    char *name;
    int (*filter)(int);
} FieldDsc;

extern FieldDsc field_dsc[NUM_FIELDS];

extern int filter_time(int C);

typedef struct _Tuple {
    char *fields[NUM_FIELDS];
    struct _Tuple *next;
    struct _Tuple *prev;
} Tuple;

extern Tuple *tuples_list;

extern Tuple *tuple_alloc(void);
extern int    tuple_maybe_unlink(Tuple **head, Tuple *tuple);
extern int    tuple_is_empty(Tuple *tuple);

/*
 * Form and form's fields description
 */
#define FORM_X_PAD          font_char_width
#define FORM_Y_PAD          font_char_width
extern int FORM_X;
#define FORM_SPACING        (2 * font_char_height)
#define FIELD_X_PAD         (font_char_width / 2)
#define FIELD_Y_PAD         (font_char_height/ 2)
#define CURSOR_Y_PAD        (font_char_height/ 2 - 1)

typedef struct _FormField {
    GdkPoint position;  /* ul corner, relative to form's ul corner */
    int      width;     /* field width (in chars, same as text width) */
} FormField;

typedef struct _Form {
    GdkDrawable *screen;   /* drawing area */
    GdkDrawable *pixmap;   /* backing pixmap */
    GdkGC       *fg_gc;    /* foreground GC */
    GdkGC       *bg_gc;    /* background GC */
    GdkPoint     position; /* ul corner, relative to drawing area's ul corner */
    Tuple       *list_head;
    Tuple       *curr_tuple;
    int          curr_field;
    int          cursor;   /* position of cursor inside curr_field field (in chars) */
    FormField   *fields;   /* array of NUM_FIELDS FormField structs */
} Form;


/*
 * Font description
 */
extern GdkFont *font;
extern int font_char_width, font_char_height;


/*
 * Form and field drawing and editing
 */
extern Form  *form_init( GdkDrawable *screen, GdkDrawable *pixmap,
                         GdkGC *fg_gc, GdkGC *bg_gc, Tuple *list_head );
extern void   form_process_char( Form *form, guint state, guint keyval,
                                 char character );

/*
 * Utilities
 */
extern const char *errMemory;
extern char  *progname;

extern void  *xmalloc(int size);
extern Tuple *read_data(FILE *fstream, char *fname);
extern void   write_data(Tuple *head, FILE *fstream);



#endif /* __BS_H__ */
