#ifndef __CLOCK_H__
#define __CLOCK_H__

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <unistd.h>
#include <time.h>
#include <gtk/gtk.h>
/* #define  NDEBUG */
#include <assert.h>

#define DEFAULT_TIMER_INTERVAL      50
/* It makes sense to make this less than 1000 because timer ticks are not
 * accurate and can arrive later or earlier; thus, we might delay clock's tick
 * by (approximately at most) REAL_TIME_TIMER_INTERVAL msec */
#define REAL_TIME_TIMER_INTERVAL    100

#define DEFAULT_HOU_RESOLUTION_NO   0
#define DEFAULT_MIN_RESOLUTION_NO   0
#define DEFAULT_SEC_RESOLUTION_NO   2 /* (SEC_RES_COUNT - 1) */

#define DEFAULT_WINDOW_WIDTH        700
#define DEFAULT_WINDOW_HEIGHT       500

#define HELP_DEFAULT_WIDTH          600
#define HELP_DEFAULT_HEIGHT         400

#define BUTTON_WIDTH_RATIO          2.5
#define BUTTON_HEIGHT_RATIO         1.2

/* Main window's widget */
extern GtkWidget *main_window;
extern GtkWidget *clock_drawing_area;
extern GtkWidget *clock_submenu;
/* Movement resolution arrays */
extern const gint sec_res_val[], min_res_val[], hou_res_val[];
extern const gint sec_res_count, min_res_count, hou_res_count;

extern GtkWidget *create_menu(GtkWidget *parent);
extern GtkWidget *create_toolbar(GtkWidget *parent);
extern void       show_resolution_dialog(GtkWidget *parent);
extern void       show_set_time_dialog(GtkWidget *parent);
extern void       show_help_dialog(GtkWidget *parent);
extern void       show_about_dialog(GtkWidget *parent);

extern gint clock_init( GtkDrawingArea *darea,
                 guint32 interval,
                 gint hou, gint min, gint sec,
                 gint hou_res_no, gint min_res_no, gint sec_res_no );
extern void clock_set_time(guint hou, guint min, guint sec);
extern void clock_get_time(guint *hou, guint *min, guint *sec);
extern void clock_set_local_time(void);
extern void clock_set_timer_interval(guint32 interval);
extern void clock_run(gboolean run);
extern void clock_set_resolutions(gint hou_res, gint min_res, gint sec_res);
extern void clock_set_resolution_indexes(gint hou_res_no, gint min_res_no, gint sec_res_no);
extern void clock_get_resolution_indexes(gint *hou, gint *min, gint *sec);
extern void clock_keep_aspect_ratio(gboolean keep);
extern void clock_run_real_time(gboolean real_time);


#endif /* __CLOCK_H__ */
