#include "clock.h"

static GtkWidget *help_window = NULL;

static gchar *help_text = \
"The clock can run in two modes:  real-time mode and `simulation' mode.  In \
real-time mode, the clock is set to local time and user settings have no \
effect.  In `simulation' mode you can adjust various setting (see below) but \
the clock will not display real system time.  To change the mode, click \
`Real-time' tool-bar button or `/Clock/Run real-time' menu item.\n\
\n\
Use accelerator keys (shown in menus), tool-bar, menu or pop-up menu (accessed \
by right-clicking clock face) to control the clock's behavior.\n\
\n\
Settings described below will take effect in `simulation' mode.\n\
\n\
Use mouse to set time displayed by the clock.  Click on the clock face with \
<Shift> and <Control> modifiers depressed to adjust hours hand; click with \
<Control> key pressed to adjust minutes hand; and click with <Shift> key \
pressed to adjust seconds hand.  Alternatively, you can use a dialog to set \
time (`Set time' tool-bar button or `/Clock/Set time' menu item).\n\
\n\
To set clock's hands to local time, click `Local time' tool-bar button \
(`/Clock/Set local time' menu item).\n\
\n\
`Interval' spinner can be used to adjust interval (in milliseconds) between \
redraws of seconds hand.\n\
\n\
Hand's movement resolution is the angle by which the hand will jerk, i.e., \
atomic rotation of the hand.  Use resolutions dialog (`Resolutions' tool-bar \
button or `/Clock/Movement resolutions' menu item) to set these for each of \
the hands.\n\
\n\
`Keep/Break A/R' tool-bar button (`/Clock/Break aspect ratio' menu item) \
allows to switch between round clock (the clock's horizontal and vertical \
radii are equal regardless of window dimensions) or elliptical clock (clock's \
horizontal and vertical radii are decided by dimensions of the window).\n\
\n\
Select font for clock face in font selection dialog (`Font' toolbar button or \
`/Clock/Select font' menu item).\n\
\n\
Pause the clock by `Stop/run' tool-bar button (`/Clock/Stop' menu item).\n\
\n\
Display information about the program by choosing `/Help/About' menu item.\n\
\n\
Access this help by clicking `Help' tool-bar button (`/Help/Clock user guide' \
menu item).\n\
\n\
Exit the program by clicking `Exit' tool-bar button (`/Clock/Exit' menu item).";

/* GTK_WIDGET(data) should be a pointer to top-level window widget */
static void ok_clicked(GtkWidget *widget, gpointer data)
{
    gtk_widget_destroy(GTK_WIDGET(data));

    return;
}

static void help_window_destroy(GtkWidget *widget, gpointer data)
{
    help_window = NULL;

    return;
}

void show_help_dialog(GtkWidget *parent)
{
    GtkWidget *window;
    GtkWidget *mainbox;
    GtkWidget *box;
    GtkWidget *widget;
    GtkWidget *text;
    GtkRequisition requisition;

    if ( help_window ) {
        gdk_window_raise(help_window->window);
        return;
    }

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_usize( window, HELP_DEFAULT_WIDTH, HELP_DEFAULT_HEIGHT );
    /* Can shrink, grow; cannot autoshrink */
    gtk_window_set_policy( GTK_WINDOW(window), TRUE, TRUE, FALSE );
    gtk_window_set_title( GTK_WINDOW(window), "Clock user guide" );
    gtk_signal_connect( GTK_OBJECT(window), "destroy",
                        GTK_SIGNAL_FUNC(help_window_destroy),
                        (gpointer) window );

    /* Top-level vbox */
    mainbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add( GTK_CONTAINER(window), mainbox );
    gtk_widget_show(mainbox);

    /* Text widget and scrollbar inside an hbox */
    box = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start( GTK_BOX(mainbox), box, TRUE, TRUE, 0 );
    gtk_widget_show(box);
    text = gtk_text_new( NULL, NULL );
    gtk_text_set_editable( GTK_TEXT(text), FALSE );
    gtk_text_set_word_wrap( GTK_TEXT(text), TRUE );
    widget = gtk_vscrollbar_new( GTK_TEXT(text)->vadj );
    gtk_box_pack_start( GTK_BOX(box), text, TRUE, TRUE, 0 );
    gtk_box_pack_start( GTK_BOX(box), widget, FALSE, FALSE, 0 );
/*     gtk_text_freeze( GTK_TEXT(text) ); */
    gtk_text_insert( GTK_TEXT(text), NULL, NULL, NULL, help_text, -1 );
/*     gtk_text_thaw( GTK_TEXT(text) ); */
    gtk_widget_show(text);
    gtk_widget_show(widget);

    /* `OK' button inside an hbox */
    box = gtk_hbox_new(TRUE, 0);
    gtk_box_pack_end( GTK_BOX(mainbox), box, FALSE, FALSE, 5 );
    widget = gtk_button_new_with_label("OK");

    gtk_box_pack_end( GTK_BOX(box), widget, FALSE, FALSE, 0 );
    gtk_signal_connect( GTK_OBJECT(widget), "clicked",
                        GTK_SIGNAL_FUNC(ok_clicked),
                        (gpointer) window );
    GTK_WIDGET_SET_FLAGS( widget, GTK_CAN_DEFAULT );
    gtk_widget_grab_default(widget);
    /* Adjust size of the button */
    gtk_widget_size_request(widget, &requisition);
    requisition.width *= BUTTON_WIDTH_RATIO;
    requisition.height *= BUTTON_HEIGHT_RATIO;
    gtk_widget_set_usize( widget, requisition.width, requisition.height );
    /* Show the button and the box */
    gtk_widget_show(widget);
    gtk_widget_show(box);

    /* Separator */
/*     widget = gtk_hseparator_new(); */
/*     gtk_box_pack_end( GTK_BOX(mainbox), widget, FALSE, FALSE, 0 ); */
/*     gtk_widget_show(widget); */

    help_window = window;

    /* Show the whole thing */
    gtk_widget_show(window);

    return;
}



/*
 * Local variables:
 * compile-command: "make -k && ./clockgtk"
 * End:
 */
