#include <X11/Xlib.h> /* Needed by focus_event handler */
#include <gdk/gdkx.h> /* Needed by focus_event handler */

#include "clock.h"
#include <gdk/gdkkeysyms.h>

#include "icon.xpm"
#include "adj_s.xpm"
#include "adj_s_mask.xpm"
#include "adj_m.xpm"
#include "adj_m_mask.xpm"
#include "adj_h.xpm"
#include "adj_h_mask.xpm"

char **about_icon_xpm; /* used by about dialog code */
GtkWidget *main_window = NULL;
GtkWidget *clock_drawing_area = NULL;
static GdkCursor *sec_adj_cursor = NULL;
static GdkCursor *min_adj_cursor = NULL;
static GdkCursor *hou_adj_cursor = NULL;

/* Close the application */
static void main_destroy(GtkWidget *widget, gpointer data)
{
    gtk_main_quit();

    return;
}

static gint focus_event(GtkWidget *widget, GdkEventFocus *event, gpointer data)
{
    if ( event->in ) {
        unsigned int  mask;
        Window        root_return, child_return;
        int           root_x_return, root_y_return;
        int           win_x_return, win_y_return;
        GdkCursor    *cursor = NULL;

        /* Some XVoodoo magic here--we are calling XQueryPointer to get the
         * state of Control and Shift keys and to set the appropriate cursor for
         * drawing area widget */
        XQueryPointer( GDK_WINDOW_XDISPLAY(GDK_ROOT_PARENT()),
                       GDK_ROOT_WINDOW(),
                       &root_return,
                       &child_return,
                       &root_x_return,
                       &root_y_return,
                       &win_x_return,
                       &win_y_return,
                       &mask );

        if ( mask & ControlMask )
            cursor = min_adj_cursor;
        /* Shift has lower priority */
        else if ( mask & ShiftMask )
            cursor = sec_adj_cursor;
        if ( cursor )
            gdk_window_set_cursor(clock_drawing_area->window, cursor);
        else
            gdk_window_set_cursor(clock_drawing_area->window, hou_adj_cursor);
    }

    return FALSE; /* pass on the event */
}

static gint key_event(GtkWidget *widget, GdkEventKey *event, gpointer data)
{
    gboolean control_key = FALSE, shift_key = FALSE;
    gboolean control_mask = FALSE, shift_mask = FALSE;
    gboolean press = FALSE, release = FALSE;
    GdkCursor *cursor = NULL;

    /* 1st button with Control modifier - adjusting minutes hand */
    /* 1st button with Shift modifier   - adjusting seconds hand */
    /* 1st button with no modifiers     - adjusting hours hand */
    /* Control has higher priority than shift */

    /* Set the flags */
    if ( event->keyval == GDK_Control_L || event->keyval == GDK_Control_R )
        control_key = TRUE;
    else if ( event->keyval == GDK_Shift_L || event->keyval == GDK_Shift_R )
        shift_key = TRUE;

    if ( event->state & GDK_CONTROL_MASK )
        control_mask = TRUE;
    if ( event->state & GDK_SHIFT_MASK )
        shift_mask = TRUE;

    if ( event->type == GDK_KEY_PRESS )
        press = TRUE;
    else
        release = TRUE;

    /* We set min_adj_cursor:
     *      whenever    control_mask == TRUE
     *      except      user has just released control key
     *      plus        user has just pressed control key
     *
     * All of the following are equivalent, but the last one makes the most
     * sense:
     */
/*     if ( (press && control_key) || (!control_key && control_mask) ) */
/*     if ( !(release && control_key) && ( control_key || control_mask ) ) */
    if ( ( control_mask && !(release && control_key) ) || ( press && control_key ) )
        cursor = min_adj_cursor;
    /* Shift has lower priority, so we set sec_adj_cursor if cursor != NULL and:
     *      whenever    and shift_mask == TRUE
     *      except      user has just released shift key
     *      plus        user has just pressed shift key
     */
    if ( !cursor )
         if ( ( shift_mask && !(release && shift_key) ) || ( press && shift_key ) )
             cursor = sec_adj_cursor;

    /* Now set the cursor */
    if ( cursor )
        gdk_window_set_cursor(GTK_WIDGET(data)->window, cursor);
    else
        gdk_window_set_cursor(GTK_WIDGET(data)->window, hou_adj_cursor);

    return FALSE; /* pass on the event */
}

static gint darea_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
    gint     sec, min, hou, angle;
    GdkPoint center;

    if ( event->button == 1 ) { /* 1st mouse button--adjust clock's hands */
        center.x = (GTK_WIDGET(data)->allocation.width  + 1) / 2;
        center.y = (GTK_WIDGET(data)->allocation.height + 1) / 2;

        /* Calculate `screen' angle (it is 90 deg ahead of `clock face' angle) */
        angle = (gint) ( 360 *
                         atan2((double) (event->y - center.y),
                               (double) (event->x - center.x))
                         / M_PI / 2 );
        /* Transform it into angle in range (0, 360] */
        angle += 180;
        /* Transform it into `clock face' angle in range [0, 360) */
        angle = (angle + 270) % 360;

        clock_get_time( &hou, &min, &sec );

        /* 1st button with Control modifier - adjusting minutes hand */
        /* 1st button with Shift modifier   - adjusting seconds hand */
        /* 1st button with no modifiers     - adjusting hours hand */
        /* Control should have higher priority than shift */
        if ( event->state & GDK_CONTROL_MASK )    /* adjust minute */
            min = rint((double) angle/(360/60));
        else if ( event->state & GDK_SHIFT_MASK ) /* adjust second */
            sec = rint((double) angle/(360/60));
        else                                      /* adjust hour */
            hou = rint((double) angle/(360/12));

        clock_set_time( hou, min, sec );

        return TRUE;
    } else if ( event->button == 2 || event->button == 3 ) { /* pop-up menu */
        gtk_menu_popup( GTK_MENU(clock_submenu), NULL, NULL, NULL, NULL,
                        event->button, event->time );

        return TRUE;
    }

    return FALSE; /* pass on the event */
}

GdkCursor *create_cursor(gchar **cursor_xpm, gchar **mask_xpm, gint x, gint y,
                         GtkWidget *parent)
{
    GdkPixmap *cursor, *mask;

    gdk_pixmap_create_from_xpm_d(
        parent->window, &cursor,
        &parent->style->bg[GTK_STATE_NORMAL],
        cursor_xpm);
    gdk_pixmap_create_from_xpm_d(
        parent->window, &mask,
        &parent->style->bg[GTK_STATE_NORMAL],
        mask_xpm);

    return gdk_cursor_new_from_pixmap(
        cursor, mask,
        gdk_color_copy(&parent->style->black),
        gdk_color_copy(&parent->style->white),
        x, y);
}



gboolean controls_run_clock(void);
int      parse_options(int argc, char **argv);
/* Options */
static gboolean autorun = TRUE;
static int main_width = DEFAULT_WINDOW_WIDTH;
static int main_height = DEFAULT_WINDOW_HEIGHT;

int main(int argc, char **argv)
{
    int i;
    GtkWidget *box;
    GtkWidget *darea;
    GdkPixmap *icon;
    GdkBitmap *mask;

    gtk_init(&argc, &argv);
    gtk_rc_parse("clockgtkrc");
    i = parse_options(argc, argv);
    if ( i == 0 ) gtk_exit(-1);
    else if ( i == 1 ) gtk_exit(0);

    about_icon_xpm = icon_xpm;

    /* Create main window */
    main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_events( main_window,
                           GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK |
                           GDK_FOCUS_CHANGE_MASK );
    gtk_signal_connect( GTK_OBJECT (main_window), "focus_in_event",
                        GTK_SIGNAL_FUNC(focus_event), NULL );
    gtk_widget_realize(main_window); /* for pixmaps */
    gtk_window_set_title( GTK_WINDOW(main_window), "Clock" );
    icon = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
                                        &main_window->style->bg[GTK_STATE_NORMAL],
                                        icon_xpm);
    gdk_window_set_icon( main_window->window, NULL, icon, mask );
    gdk_window_set_icon_name( main_window->window, "Clock" );
    gtk_container_set_border_width( GTK_CONTAINER(main_window), 0 );
    /* Can both shrik and grow */
    gtk_window_set_policy( GTK_WINDOW(main_window), TRUE, TRUE, FALSE );
    /* Make it close on destroy signal */
    gtk_signal_connect( GTK_OBJECT(main_window), "destroy",
                        GTK_SIGNAL_FUNC(main_destroy), NULL );

    /* Create vertical box (homogeneous, spacing) within main window */
    box = gtk_vbox_new(FALSE, 0);
    gtk_container_add( GTK_CONTAINER(main_window), box );
    /* Create menu */
    gtk_box_pack_start( GTK_BOX(box), create_menu(main_window), FALSE, FALSE, 0);
    /* Create toolbar */
    gtk_box_pack_start( GTK_BOX(box), create_toolbar(main_window), FALSE, FALSE, 0);
    /* Create drawing area inside the top-level box */
    darea = gtk_drawing_area_new();
    gtk_widget_set_events( darea, GDK_BUTTON_PRESS_MASK );
    gtk_signal_connect( GTK_OBJECT (darea), "button_press_event",
                        GTK_SIGNAL_FUNC(darea_button_press_event), (gpointer) darea );
    gtk_signal_connect( GTK_OBJECT (main_window), "key_press_event",
                        GTK_SIGNAL_FUNC(key_event), (gpointer) darea );
    gtk_signal_connect( GTK_OBJECT (main_window), "key_release_event",
                        GTK_SIGNAL_FUNC(key_event), (gpointer) darea );

    gtk_box_pack_start( GTK_BOX(box), darea, TRUE, TRUE, 0 );

    /* Initialize clock display */
    gtk_widget_realize(darea);
    if ( clock_init( GTK_DRAWING_AREA(darea),
                     DEFAULT_TIMER_INTERVAL, -1, -1, -1, -1, -1, -1 ) )
        gtk_exit(1);

    /* Mouse cursors */
    sec_adj_cursor =
        create_cursor(adj_s_xpm, adj_s_mask_xpm, 15, 15, darea);
    min_adj_cursor =
        create_cursor(adj_m_xpm, adj_m_mask_xpm, 15, 15, darea);
    hou_adj_cursor =
        create_cursor(adj_h_xpm, adj_h_mask_xpm, 15, 15, darea);

    clock_drawing_area = darea;

    gtk_widget_set_usize( main_window, main_width, main_height );

    /* Show all widgets */
    gtk_widget_show(darea);
    gtk_widget_show(box);
    gtk_widget_show(main_window);

    /* Set initial cursor for drawing area */
    gdk_window_set_cursor( darea->window, hou_adj_cursor );

    /* Run clock, if user did not request not to */
    if ( !autorun )
        controls_run_clock(); /* make sure state of controls is updated */
    else
        clock_run(autorun);

    /* Wait for events from X server */
    gtk_main();

    return 0;
}


char *errUnknownOption = "%s: error--unknown option `-%c'\n";
char *errUnknownOptionChar = "%s: error--unknown option character `\\x%x'\n";
char *errSizeRange     = "%s: error--size must be a positive integer\n";
char *errArgumentReq   = "%s: error--option `-%c' requires an argument (%s)\n";

char *usehelp = "Try `%s -h' for help\n";
char *usage   = "\n\
Usage: %s [-h] [-x WIDTH] [-y HEIGHT] [-s]\n\
\n\
-h        display this help message\n\
-x WIDTH  set main window's width to WIDTH   (default: %d)\n\
-y HEIGHT set main window's height to HEIGHT (default: %d)\n\
-s        do not run the clock at startup    (default: run the clock)\n\
\n";

/* Return codes:
 * 0    - error, exit(-1);
 * 1    - ok, exit(0);
 * else - ok, go on.
 */
int parse_options(int argc, char **argv)
{
    char c, *ptr;

    opterr = 0; /* prevent getopt from printing error messages */
    while ( (c = getopt(argc, argv, ":x:y:sh")) != -1 )
        switch ( c ) {
        case 'x':
            main_width = (int) strtol(optarg, &ptr, 0);
            if ( ptr == optarg ) {
                fprintf(stderr, errSizeRange, *argv, "width");
                fprintf(stderr, usehelp, *argv);
                return 0;
            }
            if ( main_width <= 0 ) {
                fprintf(stderr, errSizeRange, *argv);
                fprintf(stderr, usehelp, *argv);
                return 0;
            }
            break;
        case 'y':
            main_height = (int) strtol(optarg, &ptr, 0);
            if ( ptr == optarg ) {
                fprintf(stderr, errSizeRange, *argv, "height");
                fprintf(stderr, usehelp, *argv);
                return 0;
            }
            if ( main_height <= 0 ) {
                fprintf(stderr, errSizeRange, *argv);
                fprintf(stderr, usehelp, *argv);
                return 0;
            }
            break;
        case 's':
            autorun = FALSE;
            break;
        case 'h':
            printf(usage, *argv, DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT);
            return 1;
            break;
        case '?': /* unknown option */
            if ( isprint(optopt) ) {
                fprintf(stderr, errUnknownOption, *argv, optopt);
                fprintf(stderr, usehelp, *argv);
            }
            else {
                fprintf(stderr, errUnknownOptionChar, *argv, optopt);
                fprintf(stderr, usehelp, *argv);
            }
            return 0;
            break;
        case ':': /* missing option argument */
            switch ( optopt ) {
            case 'x':
                fprintf(stderr, errArgumentReq, *argv, optopt, "width");
                fprintf(stderr, usehelp, *argv);
                return 0;
                break;
            case 'y':
                fprintf(stderr, errArgumentReq, *argv, optopt, "height");
                fprintf(stderr, usehelp, *argv);
                return 0;
                break;
            default:
                assert(0); /* should never get here */
                abort();
                break;
            }
        default:
            assert(0); /* should never happen */
            abort();
            break;
        }

    return -1;
}



/*
 * Local variables:
 * compile-command: "make -k && ./clockgtk"
 * End:
 */
