#include "clock.h"

/* Functions controlling appearance of menu items and toolbar widgets (see
 * toolbar code) */
gboolean controls_run_clock(void);
gboolean controls_ar_keep(void);
gboolean controls_real_time(gboolean real_time);
void set_time_clicked(GtkWidget *widget, gpointer data);
void set_time_local_clicked(GtkWidget *widget, gpointer data);
void res_clicked(GtkWidget *widget, gpointer data);
void font_clicked(GtkWidget *widget, gpointer data);
void help_clicked(GtkWidget *widget, gpointer data);
void exit_clicked(GtkWidget *widget, gpointer data);

GtkWidget *clock_submenu            = NULL;

GtkWidget *menu_stop_item           = NULL;
GtkWidget *menu_break_ar_item       = NULL;
GtkWidget *menu_real_time_item      = NULL;
GtkWidget *menu_set_time_item       = NULL;
GtkWidget *menu_set_time_local_item = NULL;


static void stop_clicked( GtkWidget *widget, gpointer data )
{
    controls_run_clock();

    return;
}

static void break_ar_clicked( GtkWidget *widget, gpointer data )
{
    controls_ar_keep();

    return;
}

static void real_time_clicked( GtkWidget *widget, gpointer data )
{
    controls_real_time(GTK_CHECK_MENU_ITEM(menu_real_time_item)->active);

    return;
}

static void about_clicked( GtkWidget *widget, gpointer data )
{
    show_about_dialog(main_window);

    return;
}




static GtkItemFactoryEntry menu_items[] = {
    { "/_Clock",                     NULL, NULL,             0, "<Branch>" },
/*     { "/Clock/_Stop",                "<control>S", print_hello,      0, "<RadioItem>" }, */
/*     { "/Clock/_Run",                 "<control>R", print_hello,      0, "/Clock/Stop" }, */
/*     { "/Clock/sep" ,                 NULL,         NULL,             0, "<Separator>" }, */
    { "/Clock/_Stop",                "S",  stop_clicked,    0, "<ToggleItem>" },
    { "/Clock/_Break aspect ratio",  "B",  break_ar_clicked,0, "<ToggleItem>" },
    { "/Clock/sep",                  NULL, NULL,            0, "<Separator>" },
    { "/Clock/_Run real time",       "R",  real_time_clicked,0,"<ToggleItem>" },
    { "/Clock/Set _time",            "T",  set_time_clicked,0, NULL },
    { "/Clock/Set _local time",      "L",  set_time_local_clicked, 0, NULL },
    { "/Clock/sep",                  NULL, NULL,            0, "<Separator>" },
    { "/Clock/_Movement resolutions","M",  res_clicked,     0, NULL },
    { "/Clock/Select _font",         "F",  font_clicked,    0, NULL },
    { "/Clock/sep",                  NULL, NULL,            0, "<Separator>" },
    { "/Clock/E_xit",                "X",  exit_clicked,    0, NULL },
    { "/_Help",                      NULL, NULL,            0, "<LastBranch>" },
    { "/Help/_Clock user guide",     "F1", help_clicked,    0, NULL },
    { "/Help/_About",                "A",  about_clicked,   0, NULL },
};


GtkWidget *create_menu(GtkWidget *parent)
{
    GtkWidget *menubar;
    GtkItemFactory *item_factory;

    GtkAccelGroup *accel_group;
    gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);

    accel_group = gtk_accel_group_new();
    item_factory = gtk_item_factory_new( GTK_TYPE_MENU_BAR, "<main>",
                                         accel_group );

    gtk_item_factory_create_items( item_factory, nmenu_items, menu_items, NULL );
    gtk_window_add_accel_group (GTK_WINDOW(parent), accel_group);

    menubar = gtk_item_factory_get_widget (item_factory, "<main>");
    gtk_widget_show(menubar);

    /* Save clock menu (will be used as popup menu in drawing area) */
    clock_submenu =
        GTK_MENU_ITEM(gtk_item_factory_get_item(item_factory, "/Clock"))->submenu;

    /* Save pointers to some menu items (used by functions controlling toolbar
     * and menu widgets, see toolbar code) */
    menu_stop_item =
        gtk_item_factory_get_item(item_factory, "/Clock/Stop");
    menu_break_ar_item =
        gtk_item_factory_get_item(item_factory, "/Clock/Break aspect ratio");
    menu_real_time_item =
        gtk_item_factory_get_item(item_factory, "/Clock/Run real time");
    menu_set_time_item =
        gtk_item_factory_get_item(item_factory, "/Clock/Set time");
    menu_set_time_local_item =
        gtk_item_factory_get_item(item_factory, "/Clock/Set local time");

    return menubar;
}



/*
 * Local variables:
 * compile-command: "make -k && ./clockgtk"
 * End:
 */
