#include "clock.h"

/* Length of string representing res value */
#define RES_STR_LEN ( 2*10+3 + \
                      sizeof(combo_entry_tail) / sizeof(combo_entry_tail[0]) )

static GtkWidget *sec_combo  = NULL;
static GtkWidget *min_combo  = NULL;
static GtkWidget *hou_combo  = NULL;

static char combo_entry_tail[] = "  degrees";

/* GTK_WIDGET(data) should be a pointer to dialog widget */
static void apply_clicked(GtkWidget *widget, gpointer data)
{
    clock_set_resolutions(
        (gint) strtol( gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(hou_combo)->entry)),
                       NULL, 10 ),
        (gint) strtol( gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(min_combo)->entry)),
                       NULL, 10 ),
        (gint) strtol( gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(sec_combo)->entry)),
                       NULL, 10 ) );

    return;
}

/* GTK_WIDGET(data) should be a pointer to dialog widget */
static void set_clicked(GtkWidget *widget, gpointer data)
{
    apply_clicked(widget, data);
    gtk_widget_destroy(GTK_WIDGET(data));

    return;
}

/* GTK_WIDGET(data) should be a pointer to dialog widget */
static void cancel_clicked(GtkWidget *widget, gpointer data)
{
    gtk_widget_destroy(GTK_WIDGET(data));

    return;
}

/* Given the array of resolution values and its length, construct hbox of label
 * and combo box with entries from this array and show the widgets.  Returns the
 * constructed hbox.  If combo_ptr is not null, pointer to combo box is stored
 * in *combo_ptr */
static GtkWidget *create_combo_box(gint *res_val, gint res_count, gint set_val, gint unit,
                                   const gchar *label_text, GtkWidget **combo_ptr)
{
    int        i;
    GList     *glist;
    GtkWidget *widget;
    GtkWidget *box;
    gchar     *textbuf;

    box = gtk_hbox_new( FALSE, 10 );
    gtk_widget_show(box);
    /* Label */
    widget = gtk_label_new(label_text);
    gtk_misc_set_alignment( GTK_MISC (widget), 0, .5 );
    gtk_widget_show(widget);
    gtk_box_pack_start( GTK_BOX(box), widget, TRUE, TRUE, 0 );
    /* Combo box */
    widget = gtk_combo_new();
    gtk_combo_set_use_arrows_always( GTK_COMBO(widget), TRUE );
    gtk_entry_set_editable( GTK_ENTRY(GTK_COMBO(widget)->entry), FALSE );

    textbuf = malloc( res_count * RES_STR_LEN );
    if ( !textbuf ) {
        g_error("Memory allocation failure!\n");
        gtk_exit(1);
    }
    for ( i = 0, glist = NULL; i < res_count; i++ ) {
        sprintf( textbuf + i * RES_STR_LEN, "%d / %d%s",
                 *(res_val + i), 3600,
                 combo_entry_tail );
        glist = g_list_append(glist, textbuf + i * RES_STR_LEN);
    }
    gtk_combo_set_popdown_strings(GTK_COMBO(widget), glist);
    if ( set_val < 0 )
        set_val = 0;
    else if ( set_val >= res_count )
        set_val = res_count - 1;
    gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(widget)->entry),
                        textbuf + set_val * RES_STR_LEN );
    g_list_free(glist);
    free(textbuf);

    if ( combo_ptr ) /* store pointer to combobox */
        *combo_ptr = widget;
    gtk_widget_show(widget);
    gtk_box_pack_start( GTK_BOX(box), widget, FALSE, FALSE, 0 );

    return box;
}

void show_resolution_dialog(GtkWidget *parent)
{
    gint sec_res_no, min_res_no, hou_res_no;
    GtkWidget *dialog;
    GtkWidget *widget;
    GtkRequisition requisition;

    dialog = gtk_dialog_new();
    gtk_window_set_modal( GTK_WINDOW(dialog), TRUE );
    /* Cannot shrink, grow, or autoshrink */
    gtk_window_set_policy( GTK_WINDOW(dialog), FALSE, FALSE, FALSE );
    gtk_window_set_transient_for( GTK_WINDOW(dialog), GTK_WINDOW(parent) );
    gtk_window_set_title( GTK_WINDOW(dialog), "Set movement resolutions" );
    gtk_box_set_homogeneous( GTK_BOX(GTK_DIALOG(dialog)->action_area), TRUE );
    gtk_container_set_border_width( GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), 0 );
    gtk_box_set_spacing( GTK_BOX(GTK_DIALOG(dialog)->action_area), 10 );
    gtk_container_set_border_width( GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 10 );
    gtk_box_set_spacing( GTK_BOX(GTK_DIALOG(dialog)->vbox), 10 );

    /* Combo boxes */
    clock_get_resolution_indexes( &hou_res_no, &min_res_no, &sec_res_no );
    widget = create_combo_box( (gint *) hou_res_val, hou_res_count, hou_res_no, 300,
                               "Hours hand:", &hou_combo );
    gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->vbox), widget,
                        TRUE, FALSE, 0 );

    widget = create_combo_box( (gint *) min_res_val, min_res_count, min_res_no, 60,
                               "Minutes hand:", &min_combo );
    gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->vbox), widget,
                        TRUE, FALSE, 0 );

    widget = create_combo_box( (gint *) sec_res_val, sec_res_count, sec_res_no, 60,
                               "Seconds hand:", &sec_combo );
    gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->vbox), widget,
                        TRUE, FALSE, 0 );

    /* `Set' button */
    widget = gtk_button_new_with_label("Set");
    gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->action_area), widget, TRUE, FALSE, 0 );
    gtk_signal_connect( GTK_OBJECT(widget), "clicked",
                        GTK_SIGNAL_FUNC(set_clicked),
                        (gpointer) dialog );
    GTK_WIDGET_SET_FLAGS( widget, GTK_CAN_DEFAULT );
    /* Get size of the button and adjust its dimensions */
    gtk_widget_size_request(widget, &requisition);
    requisition.width *= BUTTON_WIDTH_RATIO;
    requisition.height *= BUTTON_HEIGHT_RATIO;
    gtk_widget_set_usize( widget, requisition.width, requisition.height );
    gtk_widget_show(widget);

    /* `Apply' button */
    widget = gtk_button_new_with_label("Apply");
    gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->action_area), widget, TRUE, FALSE, 0 );
    gtk_signal_connect( GTK_OBJECT(widget), "clicked",
                        GTK_SIGNAL_FUNC(apply_clicked),
                        (gpointer) dialog );
    GTK_WIDGET_SET_FLAGS( widget, GTK_CAN_DEFAULT );
    gtk_widget_set_usize( widget, requisition.width, requisition.height );
    gtk_widget_show(widget);

    /* `Cancel' button */
    widget = gtk_button_new_with_label("Cancel");
    gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->action_area), widget, TRUE, FALSE, 0 );
    gtk_signal_connect( GTK_OBJECT(widget), "clicked",
                        GTK_SIGNAL_FUNC(cancel_clicked),
                        (gpointer) dialog );
    GTK_WIDGET_SET_FLAGS( widget, GTK_CAN_DEFAULT );
    gtk_widget_grab_default(widget);
    gtk_widget_set_usize( widget, requisition.width, requisition.height );
    gtk_widget_show(widget);

    gtk_widget_show(dialog);

    return;
}



/*
 * Local variables:
 * compile-command: "make -k && ./clockgtk"
 * End:
 */
