#include "clock.h"

static GtkWidget *sec_spin = NULL;
static GtkWidget *min_spin = NULL;
static GtkWidget *hou_spin = NULL;


/* GTK_WIDGET(data) should be a pointer to dialog widget */
static void apply_clicked(GtkWidget *widget, gpointer data)
{
    clock_set_time(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(hou_spin)),
                   gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(min_spin)),
                   gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(sec_spin)));

    return;
}

/* GTK_WIDGET(data) should be a pointer to dialog widget */
static void set_clicked(GtkWidget *widget, gpointer data)
{
    apply_clicked(widget, data);
    gtk_widget_destroy(GTK_WIDGET(data));

    return;
}

/* GTK_WIDGET(data) should be a pointer to dialog widget */
static void cancel_clicked(GtkWidget *widget, gpointer data)
{
    gtk_widget_destroy(GTK_WIDGET(data));

    return;
}

/* Construct hbox of label and spin button and show the widgets.  Returns the
 * constructed hbox.  If spin_ptr is not null, pointer to spin button is stored
 * in *spin_ptr */
static GtkWidget *create_spin_button(gint max, gchar *label_text, gint set_val,
                                     gint page_increment, GtkWidget **spin_ptr)
{
    GtkWidget *widget;
    GtkWidget *box;

    box = gtk_hbox_new( FALSE, 0 );
    /* Label */
    widget = gtk_label_new(label_text);
    gtk_misc_set_alignment( GTK_MISC (widget), 0, .5 );
    gtk_widget_show(widget);
    gtk_box_pack_start( GTK_BOX(box), widget, TRUE, FALSE, 5 );
    /* Spin button */
    widget = (GtkWidget *) gtk_adjustment_new( set_val, 0.0, (gfloat) max, 1.0,
                                               (gfloat) page_increment, 0.0 );
    widget = gtk_spin_button_new(
        GTK_ADJUSTMENT( widget ), .5, 0 ); /* (climb_rate, digits) */
    gtk_spin_button_set_wrap( GTK_SPIN_BUTTON(widget), TRUE );
    gtk_widget_show(widget);
    gtk_box_pack_start( GTK_BOX(box), widget, TRUE, FALSE, 0 );
    gtk_widget_show(box);

    if ( spin_ptr )
        *spin_ptr = widget;

    return box;
}

void show_set_time_dialog(GtkWidget *parent)
{
    gint       sec, min, hou;
    GtkWidget *dialog;
    GtkWidget *widget;
    GtkWidget *box;
    GtkRequisition requisition;

    dialog = gtk_dialog_new();
    gtk_window_set_modal( GTK_WINDOW(dialog), TRUE );
    /* Cannot shrink, grow, or autoshrink */
    gtk_window_set_policy( GTK_WINDOW(dialog), FALSE, FALSE, FALSE );
    gtk_window_set_transient_for( GTK_WINDOW(dialog), GTK_WINDOW(parent) );
    gtk_window_set_title( GTK_WINDOW(dialog), "Set time" );
    gtk_box_set_homogeneous( GTK_BOX(GTK_DIALOG(dialog)->action_area), TRUE );
    gtk_container_set_border_width( GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), 0 );
    gtk_box_set_spacing( GTK_BOX(GTK_DIALOG(dialog)->action_area), 10 );
    gtk_container_set_border_width( GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 10 );
    gtk_box_set_spacing( GTK_BOX(GTK_DIALOG(dialog)->vbox), 10 );

    /* hbox in which spin buttons will be packed */
    box = gtk_hbox_new( FALSE, 10 );

    /* Spin buttons */
    clock_get_time( &hou, &min, &sec );

    widget = create_spin_button( 11, "H:", hou, 4, &hou_spin );
    gtk_box_pack_start( GTK_BOX(box), widget, TRUE, TRUE, 0 );

    widget = create_spin_button( 59, "M:", min, 10, &min_spin );
    gtk_box_pack_start( GTK_BOX(box), widget, TRUE, TRUE, 0 );

    widget = create_spin_button( 59, "S:", sec, 10, &sec_spin );
    gtk_box_pack_start( GTK_BOX(box), widget, TRUE, TRUE, 0 );

    gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->vbox), box, TRUE, TRUE, 0 );
    gtk_widget_show(box);

    /* `Set' button */
    widget = gtk_button_new_with_label("Set");
    gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->action_area), widget, TRUE, TRUE, 0 );
    gtk_signal_connect( GTK_OBJECT(widget), "clicked",
                        GTK_SIGNAL_FUNC(set_clicked),
                        (gpointer) dialog );
    GTK_WIDGET_SET_FLAGS( widget, GTK_CAN_DEFAULT );
    /* Get size of the button and adjust its dimensions */
    gtk_widget_size_request(widget, &requisition);
    requisition.width *= BUTTON_WIDTH_RATIO;
    requisition.height *= BUTTON_HEIGHT_RATIO;
    gtk_widget_set_usize( widget, requisition.width, requisition.height );
    gtk_widget_show(widget);

    /* `Apply' button */
    widget = gtk_button_new_with_label("Apply");
    gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->action_area), widget, TRUE, TRUE, 0 );
    gtk_signal_connect( GTK_OBJECT(widget), "clicked",
                        GTK_SIGNAL_FUNC(apply_clicked),
                        (gpointer) dialog );
    GTK_WIDGET_SET_FLAGS( widget, GTK_CAN_DEFAULT );
    gtk_widget_set_usize( widget, requisition.width, requisition.height );
    gtk_widget_show(widget);

    /* `Cancel' button */
    widget = gtk_button_new_with_label("Cancel");
    gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->action_area), widget, TRUE, TRUE, 0 );
    gtk_signal_connect( GTK_OBJECT(widget), "clicked",
                        GTK_SIGNAL_FUNC(cancel_clicked),
                        (gpointer) dialog );
    GTK_WIDGET_SET_FLAGS( widget, GTK_CAN_DEFAULT );
    gtk_widget_grab_default(widget);
    gtk_widget_set_usize( widget, requisition.width, requisition.height );
    gtk_widget_show(widget);

    gtk_widget_show(dialog);

    return;
}



/*
 * Local variables:
 * compile-command: "make -k && ./clockgtk"
 * End:
 */
