#include "clock.h"

#include "run.xpm"
#include "stop.xpm"
#include "ar_keep.xpm"
#include "ar_break.xpm"
#include "set_time.xpm"
#include "set_time_local.xpm"
#include "real_time.xpm"
#include "res.xpm"
#include "font.xpm"
#include "help.xpm"
#include "exit.xpm"


/* This are menu items used by handlers controlling their appearance */
extern GtkWidget *menu_stop_item, *menu_break_ar_item, *menu_real_time_item;
extern GtkWidget *menu_set_time_item, *menu_set_time_local_item;


static GtkFontSelectionDialog *fsdialog = NULL;

static gboolean   toolbar_initialized = FALSE;

static GtkWidget *toolbar             = NULL;

static GtkWidget *run_stop_button     = NULL;
static GtkWidget *run_button_stop_box = NULL;
static GtkWidget *run_button_run_box  = NULL;
static gchar *stop_tooltip = "Stop the clock";
static gchar *run_tooltip  = "Run the clock";
static gchar *run_button_tooltip_private = "Run/stop";

static GtkWidget *keep_ar_button           = NULL;
static GtkWidget *ar_keep_button_keep_box  = NULL;
static GtkWidget *ar_keep_button_break_box = NULL;
static gchar *ar_keep_tooltip  = "Keep aspect ratio";
static gchar *ar_break_tooltip = "Break aspect ratio";
static gchar *ar_keep_button_tooltip_private  = "Keep/break A/R";

static GtkWidget *real_time_button         = NULL;

static GtkWidget *timer_interval_box       = NULL;
static GtkWidget *timer_interval_spinner   = NULL;
static GtkWidget *set_time_button          = NULL;
static GtkWidget *set_time_local_button    = NULL;


/* Called when toolbar widget or menu item is clicked to uniformly set states */
gboolean controls_run_clock(void)
{
    static gboolean run = TRUE; /* initially clock runs */

    run = !run;                 /* invert state */

    /* Toolbar button is two-faced, so show one face and hide another */
    if ( run ) { /* making the clock run */
        gtk_widget_hide( run_button_run_box );
        gtk_tooltips_set_tip( GTK_TOOLBAR(toolbar)->tooltips,
                              run_stop_button,
                              stop_tooltip,
                              run_button_tooltip_private );
        gtk_widget_show( run_button_stop_box );
    } else {     /* stopping the clock */
        gtk_widget_hide( run_button_stop_box );
        gtk_tooltips_set_tip( GTK_TOOLBAR(toolbar)->tooltips,
                              run_stop_button,
                              run_tooltip,
                              run_button_tooltip_private );
        gtk_widget_show( run_button_run_box );
    }

    /* Configure menu item */
    GTK_CHECK_MENU_ITEM(menu_stop_item)->active = !run;
    /* Run or stop the clock */
    clock_run(run);
    /* Button's width might have changed, so resize toolbar */
    gtk_widget_queue_resize(toolbar);

    return run;
}

void run_button_clicked(GtkWidget *widget, gpointer data)
{
    controls_run_clock();

    return;
}

/* Called when toolbar widget or menu item is clicked to uniformly set states */
gboolean controls_ar_keep(void)
{
    static gboolean keep = TRUE;

    keep = !keep;
   /* This button is two-faced, so show one face and hide another */
    if ( keep ) { /* keeping a/r */
        gtk_widget_hide( ar_keep_button_keep_box );
        gtk_tooltips_set_tip( GTK_TOOLBAR(toolbar)->tooltips, keep_ar_button,
                              ar_break_tooltip, ar_keep_button_tooltip_private );
        gtk_widget_show( ar_keep_button_break_box );
    } else {      /* breaking a/r */
        gtk_widget_hide( ar_keep_button_break_box );
        gtk_tooltips_set_tip( GTK_TOOLBAR(toolbar)->tooltips, keep_ar_button,
                              ar_keep_tooltip, ar_keep_button_tooltip_private );
        gtk_widget_show( ar_keep_button_keep_box );
    }

    /* Configure menu item */
    GTK_CHECK_MENU_ITEM(menu_break_ar_item)->active = !keep;
    /* Keep or break a/r */
    clock_keep_aspect_ratio(keep);
    /* Button's width might have changed, so resize toolbar */
    gtk_widget_queue_resize(toolbar);

    return keep;
}

static void ar_keep_button_clicked(GtkWidget *widget, gpointer data)
{
    controls_ar_keep();

    return;
}

/* Called when toolbar widget or menu item is clicked to uniformly set states */
gboolean controls_real_time(gboolean real_time)
{
    gtk_widget_set_sensitive( set_time_button, !real_time );
    gtk_widget_set_sensitive( set_time_local_button, !real_time );
    gtk_widget_set_sensitive( timer_interval_box, !real_time );
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(real_time_button), real_time);

    gtk_widget_set_sensitive( menu_set_time_item, !real_time );
    gtk_widget_set_sensitive( menu_set_time_local_item, !real_time );
    GTK_CHECK_MENU_ITEM(menu_real_time_item)->active = real_time;

    clock_run_real_time( real_time );

    return real_time;
}

static void real_time_toggled(GtkWidget *widget, gpointer data)
{
    controls_real_time(GTK_TOGGLE_BUTTON(widget)->active);

    return;
}

void set_time_clicked(GtkWidget *widget, gpointer data)
{
    show_set_time_dialog(main_window);

    return;
}

void set_time_local_clicked(GtkWidget *widget, gpointer data)
{
    clock_set_local_time();

    return;
}

static void interval_value_changed(GtkWidget *widget, gpointer data)
{
    if ( toolbar_initialized )
        clock_set_timer_interval( (guint32) GTK_ADJUSTMENT(widget)->value );

    return;
}

void res_clicked(GtkWidget *widget, gpointer data)
{
    show_resolution_dialog(main_window);

    return;
}

static void fsdialog_apply_clicked(GtkWidget *widget, gpointer data)
{
    GdkFont  *font;
    GtkStyle *style;

    font = gtk_font_selection_dialog_get_font(GTK_FONT_SELECTION_DIALOG(data));
    if ( font ) {
        style = gtk_style_copy(gtk_widget_get_style(clock_drawing_area));
        gdk_font_unref(style->font);
        gdk_font_ref(font);
        style->font = font;
        gtk_widget_set_style(clock_drawing_area, style);
        gtk_widget_queue_resize(clock_drawing_area);
    }

    return;
}

static void fsdialog_ok_clicked(GtkWidget *widget, gpointer data)
{
    fsdialog_apply_clicked(widget, data);
    gtk_widget_destroy(GTK_WIDGET(data));

    return;
}

static void fsdialog_cancel_clicked(GtkWidget *widget, gpointer data)
{
    gtk_widget_destroy(GTK_WIDGET(data));

    return;
}

static void fsdialog_destroy(GtkWidget *widget, gpointer data)
{
    fsdialog = NULL;

    return;
}

void font_clicked(GtkWidget *widget, gpointer data)
{
    if ( fsdialog ) {
        gdk_window_raise(GTK_WIDGET(fsdialog)->window);
        return;
    }

    fsdialog = GTK_FONT_SELECTION_DIALOG(
        gtk_font_selection_dialog_new("Select font for the clock"));
/*     gtk_window_set_transient_for( GTK_WINDOW(fsdialog), GTK_WINDOW(data) ); */
    gtk_widget_show(fsdialog->apply_button);
    gtk_signal_connect( GTK_OBJECT(fsdialog->ok_button), "clicked",
                        GTK_SIGNAL_FUNC(fsdialog_ok_clicked),
                        (gpointer) fsdialog );
    gtk_signal_connect( GTK_OBJECT(fsdialog->apply_button), "clicked",
                        GTK_SIGNAL_FUNC(fsdialog_apply_clicked),
                        (gpointer) fsdialog );
    gtk_signal_connect( GTK_OBJECT(fsdialog->cancel_button), "clicked",
                        GTK_SIGNAL_FUNC(fsdialog_cancel_clicked),
                        (gpointer) fsdialog );
    gtk_signal_connect( GTK_OBJECT(fsdialog), "destroy",
                        GTK_SIGNAL_FUNC(fsdialog_destroy),
                        (gpointer) fsdialog );
    gtk_font_selection_dialog_set_preview_text(
        fsdialog, "1 2 3 4 5 6 7 8 9 10 11 12" );

    gtk_widget_show(GTK_WIDGET(fsdialog));

    return;
}

void help_clicked(GtkWidget *widget, gpointer data)
{
    show_help_dialog(main_window);

    return;
}

void exit_clicked(GtkWidget *widget, gpointer data)
{
    gtk_widget_destroy(main_window);

    return;
}

/* Create a new vbox with an image and a label packed into it and return the
 * box */
static GtkWidget *xpm_d_label_vbox(GtkWidget *parent, gchar **xpm_d, gchar *ltext)
{
    GtkWidget *box;
    GdkPixmap *icon;
    GdkBitmap *mask;
    GtkWidget *widget;

    box  = gtk_vbox_new(FALSE, 0);
    icon = gdk_pixmap_create_from_xpm_d(parent->window, &mask,
                                        &parent->style->bg[GTK_STATE_NORMAL],
                                        xpm_d);
    widget = gtk_pixmap_new(icon, mask);
    gtk_box_pack_start( GTK_BOX(box), widget, TRUE, FALSE, 0 );
    gtk_widget_show(widget);
    widget = gtk_label_new(ltext);
    gtk_box_pack_start( GTK_BOX(box), widget, TRUE, FALSE, 0 );
    gtk_widget_show(widget);

    return box;
}

/* Create and show toolbar.  Pass parent widget, which should be shown or
 * realized before call to this function.  Returns the created toolbar widget */
GtkWidget *create_toolbar(GtkWidget *parent)
{
    GtkWidget *tbar;
    GtkWidget *handlebox;
    GtkWidget *widget;
    GtkWidget *box;
    GtkWidget *eventbox;
    GdkPixmap *icon;
    GdkBitmap *mask;

    toolbar_initialized = FALSE;

    /*
     * Handlebox and toolbar
     */
    handlebox = gtk_handle_box_new();
    tbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
    gtk_container_set_border_width( GTK_CONTAINER(tbar), 2 );
    gtk_toolbar_set_space_size( GTK_TOOLBAR(tbar), 0 );
    gtk_toolbar_set_button_relief( GTK_TOOLBAR(tbar), GTK_RELIEF_NONE );/*???*/
    gtk_container_add( GTK_CONTAINER(handlebox), tbar );

    /*
     * `Run/stop' two-faced button
     */
    /* This button will have two `faces', so create both of them as vboxes and
     * pack into one hbox */
    run_button_stop_box = xpm_d_label_vbox(parent, stop_xpm, "Stop");
    run_button_run_box  = xpm_d_label_vbox(parent, run_xpm,  "Run");
    box = gtk_hbox_new( FALSE, 0 );
    gtk_box_pack_start( GTK_BOX(box), run_button_stop_box, TRUE, FALSE, 0 );
    gtk_box_pack_start( GTK_BOX(box), run_button_run_box, TRUE, FALSE, 0 );
    gtk_widget_show(run_button_stop_box);
    gtk_widget_show(box);
    /* Create toolbar button with default `face' */
    widget = gtk_toolbar_append_item ( GTK_TOOLBAR(tbar),
                                       NULL,
                                       stop_tooltip,
                                       run_button_tooltip_private,
                                       NULL,
                                       GTK_SIGNAL_FUNC(run_button_clicked),
                                       (gpointer) tbar );
    run_stop_button = widget;
    /* Kill the button's child and insert our hbox instead */
    gtk_widget_destroy(GTK_BIN(widget)->child);
    gtk_container_add( GTK_CONTAINER (widget), box );
    gtk_toolbar_append_space( GTK_TOOLBAR(tbar) );

    /*
     * `Keep/break aspect ratio' two-faced button
     */
    /* This button will have two `faces', so create both of them as vboxes and
     * pack into one hbox */
    ar_keep_button_keep_box  = xpm_d_label_vbox(parent, ar_keep_xpm,  "Keep A / R");
    ar_keep_button_break_box = xpm_d_label_vbox(parent, ar_break_xpm, "Break A / R");
    box = gtk_hbox_new( FALSE, 0 );
    gtk_box_pack_start( GTK_BOX(box), ar_keep_button_break_box, TRUE, FALSE, 0 );
    gtk_box_pack_start( GTK_BOX(box), ar_keep_button_keep_box, TRUE, FALSE, 0 );
    gtk_widget_show(ar_keep_button_break_box);
    gtk_widget_show(box);
    /* Create toolbar button with default `face' */
    widget = gtk_toolbar_append_item ( GTK_TOOLBAR(tbar),
                                       NULL,
                                       ar_break_tooltip,
                                       ar_keep_button_tooltip_private,
                                       NULL,
                                       GTK_SIGNAL_FUNC(ar_keep_button_clicked),
                                       (gpointer) tbar );
    keep_ar_button = widget;
    /* Kill the button's child and insert our hbox instead */
    gtk_widget_destroy(GTK_BIN(widget)->child );
    gtk_container_add( GTK_CONTAINER (widget), box );
    gtk_toolbar_append_space( GTK_TOOLBAR(tbar) );

    /*
     * `Run real-time' button
     */
    icon = gdk_pixmap_create_from_xpm_d(parent->window, &mask,
                                        &tbar->style->bg[GTK_STATE_NORMAL],
                                        real_time_xpm);
    widget = gtk_pixmap_new(icon, mask);
    real_time_button = gtk_toolbar_append_element (
        GTK_TOOLBAR(tbar),
        GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        NULL,
        "Real-time",
        "Run the clock real-time",
        "Real-time",
        widget,
        GTK_SIGNAL_FUNC(real_time_toggled),
        NULL );
    gtk_toolbar_append_space( GTK_TOOLBAR(tbar) );

    /*
     * `Set time' button
     */
    icon = gdk_pixmap_create_from_xpm_d(parent->window, &mask,
                                        &tbar->style->bg[GTK_STATE_NORMAL],
                                        set_time_xpm);
    widget = gtk_pixmap_new(icon, mask);
    set_time_button = gtk_toolbar_append_item (
        GTK_TOOLBAR(tbar),
        "Set time",
        "Set time displayed by the clock",
        "Set time",
        widget,
        GTK_SIGNAL_FUNC(set_time_clicked),
        NULL );
    gtk_toolbar_append_space( GTK_TOOLBAR(tbar) );

    /*
     * `Set local time' button
     */
    icon = gdk_pixmap_create_from_xpm_d(parent->window, &mask,
                                        &tbar->style->bg[GTK_STATE_NORMAL],
                                        set_time_local_xpm);
    widget = gtk_pixmap_new(icon, mask);
    set_time_local_button = gtk_toolbar_append_item (
        GTK_TOOLBAR(tbar),
        "Local time",
        "Set the clock to the local time",
        "time local",
        widget,
        GTK_SIGNAL_FUNC(set_time_local_clicked),
        NULL );
    gtk_toolbar_append_space( GTK_TOOLBAR(tbar) );

    /*
     * Create `tick delay' spin button
     */
    eventbox = gtk_event_box_new();
    box = gtk_vbox_new(FALSE, 10);
    widget = gtk_label_new("Interval");
    gtk_widget_show(widget);
    gtk_box_pack_start( GTK_BOX(box), widget, TRUE, TRUE, 0 );
    widget = (GtkWidget *) gtk_adjustment_new(DEFAULT_TIMER_INTERVAL,
                                              1.0, 1000.0, 1.0, 100.0, 0.0);
    gtk_signal_connect( GTK_OBJECT (widget), "value_changed",
                        GTK_SIGNAL_FUNC (interval_value_changed),
                        (gpointer) NULL);
    widget = gtk_spin_button_new(
        GTK_ADJUSTMENT( widget ), 1.0, 0 ); /* (climb_rate, digits) */
    timer_interval_spinner = widget;
    gtk_widget_show(widget);
    gtk_box_pack_start( GTK_BOX(box), widget, TRUE, TRUE, 0 );
    gtk_widget_show(box);
    gtk_container_add(GTK_CONTAINER(eventbox), box);
    gtk_widget_show(eventbox);
    gtk_toolbar_append_widget( GTK_TOOLBAR(tbar),
                               eventbox,
                               "Adjust timer interval (msec)",
                               "Timer interval");
    gtk_toolbar_append_space( GTK_TOOLBAR(tbar) );
    timer_interval_box = box;

    /*
     * `Movement resolutions' button
     */
    icon = gdk_pixmap_create_from_xpm_d(parent->window, &mask,
                                        &tbar->style->bg[GTK_STATE_NORMAL],
                                        res_xpm);
    widget = gtk_pixmap_new(icon, mask);
    gtk_toolbar_append_item ( GTK_TOOLBAR(tbar),
                              "Resolutions",
                              "Set movement resolutions of the hands",
                              "Resolutions",
                              widget,
                              GTK_SIGNAL_FUNC(res_clicked),
                              NULL );
    gtk_toolbar_append_space( GTK_TOOLBAR(tbar) );

    /*
     * `Font selection' button
     */
    icon = gdk_pixmap_create_from_xpm_d(parent->window, &mask,
                                        &tbar->style->bg[GTK_STATE_NORMAL],
                                        font_xpm);
    widget = gtk_pixmap_new(icon, mask);
    gtk_toolbar_append_item ( GTK_TOOLBAR(tbar),
                              "Font",
                              "Select font for the clock",
                              "Font",
                              widget,
                              GTK_SIGNAL_FUNC(font_clicked),
                              (gpointer) parent );

    /*
     * `Help' button
     */
    icon = gdk_pixmap_create_from_xpm_d(parent->window, &mask,
                                        &tbar->style->bg[GTK_STATE_NORMAL],
                                        help_xpm);
    widget = gtk_pixmap_new(icon, mask);
    gtk_toolbar_append_item ( GTK_TOOLBAR(tbar),
                              "Help",
                              "Display help",
                              "Help",
                              widget,
                              GTK_SIGNAL_FUNC(help_clicked),
                              NULL );

    /*
     * `Exit' button
     */
    icon = gdk_pixmap_create_from_xpm_d(parent->window, &mask,
                                        &tbar->style->bg[GTK_STATE_NORMAL],
                                        exit_xpm);
    widget = gtk_pixmap_new(icon, mask);
    gtk_toolbar_append_item ( GTK_TOOLBAR(tbar),
                              "Exit",
                              "Exit the clock",
                              "Exit",
                              widget,
                              GTK_SIGNAL_FUNC(exit_clicked),
                              NULL );

    toolbar = tbar;
    toolbar_initialized = TRUE;

    gtk_widget_show(tbar);
    gtk_widget_show(handlebox);

    return handlebox;
}



/*
 * Local variables:
 * compile-command: "make -k && ./clockgtk"
 * End:
 */
