#include "kybos.h"
#include <stdlib.h>
#include <math.h>
#include <assert.h>


Point *recalc_into_viewcs( Kybos *obj )
{
    Point *viewvertices;
    int i;
    double cos_phi, sin_phi, cos_theta, sin_theta;
    double proj_xy, ro;

    assert(obj);
    viewvertices = (Point *) xmalloc(sizeof(Point) * obj->num_vertices);

    /* Recalculate Cartesian coordinates of the viewpoint into spherical
       coordinates */
    ro = sqrt( obj->viewpoint.x * obj->viewpoint.x +
               obj->viewpoint.y * obj->viewpoint.y +
               obj->viewpoint.z * obj->viewpoint.z );
    proj_xy = sqrt( obj->viewpoint.x * obj->viewpoint.x +
                    obj->viewpoint.y * obj->viewpoint.y );
    cos_phi   = obj->viewpoint.z / ro;
    sin_phi   = proj_xy / ro;
    cos_theta = obj->viewpoint.x / proj_xy;
    sin_theta = obj->viewpoint.y / proj_xy;
    /* Recalculate world coordinates of the object into the view coordinates */
    for ( i = 0; i < obj->num_vertices; i++ ) {
        double x = (obj->vertices + i)->x;
        double y = (obj->vertices + i)->y;
        double z = (obj->vertices + i)->z;
        (viewvertices + i)->x =
            -x * sin_theta
            + y * cos_theta;
        (viewvertices + i)->y =
            - x * cos_phi * cos_theta
            - y * cos_phi * sin_theta
            + z * sin_phi;
        (viewvertices + i)->z =
            - x * sin_phi * cos_theta
            - y * sin_phi * sin_theta
            - z * cos_phi + ro;
    }

    return viewvertices;
}


#define IS_VISIBLE_PERSPECTIVE(P1, P2, P3)                  \
                    ( (P1)->x * (P2)->y * (P3)->z +         \
                      (P2)->x * (P3)->y * (P1)->z +         \
                      (P3)->x * (P1)->y * (P2)->z -         \
                      (P3)->x * (P2)->y * (P1)->z -         \
                      (P2)->x * (P1)->y * (P3)->z -         \
                      (P1)->x * (P3)->y * (P2)->z   > 0 )

#define IS_VISIBLE_PARALLEL(P1, P2, P3)                                     \
                    ( ( (P2)->x - (P1)->x ) * ( (P3)->y - (P1)->y ) -       \
                      ( (P2)->y - (P1)->y ) * ( (P3)->x - (P1)->x )   > 0 )

void draw_object( GdkDrawable *drawable, Kybos *obj )
{
    Point    *viewvertices;
    Triangle *curr;
    GdkPoint *triang;
    gint      width, height;

    assert(obj);
    viewvertices = recalc_into_viewcs( obj );
    triang       = (GdkPoint *) xmalloc(sizeof(GdkPoint) * 3);

    gdk_window_get_size( drawable, &width, &height );
    width  /= 2;
    height /= 2;

    curr = obj->triangles;
    if ( obj->projection )  /* doing perspective projection */
        while ( curr ) {
            Point *p1 = viewvertices + curr->p1;
            Point *p2 = viewvertices + curr->p2;
            Point *p3 = viewvertices + curr->p3;
            /* Flip the picture vertically (hence `-' for the y coordinate) */
            if ( IS_VISIBLE_PERSPECTIVE(p1, p2, p3) ) {
                (triang+0)->x =  obj->screen * p1->x / p1->z + width;
                (triang+0)->y = -obj->screen * p1->y / p1->z + height;
                (triang+1)->x =  obj->screen * p2->x / p2->z + width;
                (triang+1)->y = -obj->screen * p2->y / p2->z + height;
                (triang+2)->x =  obj->screen * p3->x / p3->z + width;
                (triang+2)->y = -obj->screen * p3->y / p3->z + height;
                gdk_draw_polygon(drawable, *(obj->gcs + curr->color), TRUE, triang, 3);
            }
            curr = curr->next;
        }
    else                    /* doing parallel projection */
        while ( curr ) {
            Point *p1 = viewvertices + curr->p1;
            Point *p2 = viewvertices + curr->p2;
            Point *p3 = viewvertices + curr->p3;
            /* Flip the picture vertically (hence `-' for the y coordinate) */
            if ( IS_VISIBLE_PARALLEL(p1, p2, p3) ) {
                (triang+0)->x =  p1->x + width;
                (triang+0)->y = -p1->y + height;
                (triang+1)->x =  p2->x + width;
                (triang+1)->y = -p2->y + height;
                (triang+2)->x =  p3->x + width;
                (triang+2)->y = -p3->y + height;
                gdk_draw_polygon(drawable, *(obj->gcs + curr->color), TRUE, triang, 3);
            }
            curr = curr->next;
        }


    free(triang);
    free(viewvertices);

    return;
}



/*
 * Local variables:
 * compile-command:  "make -k && ./kybos < kybos.data"
 * End:
 */
