#include "kybos.h"
#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>


char *progname = NULL;


FILE *stream_in  = NULL;
int   projection = 1;
static GtkWidget *main_window   = NULL;
static GtkWidget *drawing_area  = NULL;
static GdkPixmap *backing_store = NULL;
static Kybos *obj = NULL;


/* Close the application */
static void main_destroy(GtkWidget *widget, gpointer data)
{
    gtk_main_quit();

    return;
}

static gint key_event(GtkWidget *widget, GdkEventKey *event, gpointer data)
{
    int redraw = 1;

    switch ( event->keyval ) {
        /*
         * Move view coordinate system along world y-axis
         */
    case GDK_Right: case GDK_KP_Right:
        obj->viewpoint.y += obj->incr_y;
        break;
    case GDK_Left: case GDK_KP_Left:
        obj->viewpoint.y -= obj->incr_y;
        break;
        /*
         * Move view coordinate system along world z-axis
         */
    case GDK_Up: case GDK_KP_Up:
        obj->viewpoint.z += obj->incr_z;
        break;
    case GDK_Down: case GDK_KP_Down:
        obj->viewpoint.z -= obj->incr_z;
        break;
        /*
         * Move view coordinate system along world x-axis
         */
    case GDK_End: case GDK_KP_End:
        obj->viewpoint.x += obj->incr_x;
        break;
    case GDK_Page_Down: case GDK_KP_Page_Down:
        obj->viewpoint.x -= obj->incr_x;
        break;
        /*
         * Move screen along view coordinate's z-axis
         */
    case GDK_Home: case GDK_KP_Home:
        if ( obj->projection ) {
            obj->screen -= obj->incr_s;
            if ( obj->screen < 0 )
                obj->screen = 0;
        } else
            redraw = 0;
        break;
    case GDK_Page_Up: case GDK_KP_Page_Up:
        if ( obj->projection )
            obj->screen += obj->incr_s;
        else
            redraw = 0;
        break;
        /*
         * Quit
         */
    case GDK_Q: case GDK_q: case GDK_Escape:
        gtk_exit(0);
        break;
    default:
        redraw = 0;
        break;
    }

    if ( redraw ) {
        /* Do the backing store */
        gdk_draw_rectangle( backing_store,
                            drawing_area->style->bg_gc[GTK_WIDGET_STATE(drawing_area)],
                            TRUE, 0, 0,
                            drawing_area->allocation.width,
                            drawing_area->allocation.height);
        draw_object( (GdkDrawable *) backing_store, obj );
        /* Do the screen */
        gtk_widget_queue_draw(drawing_area);
/*         gdk_draw_rectangle( drawing_area->window, */
/*                             drawing_area->style->bg_gc[GTK_WIDGET_STATE(drawing_area)], */
/*                             TRUE, 0, 0, */
/*                             drawing_area->allocation.width, */
/*                             drawing_area->allocation.height); */
/*         draw_object( (GdkDrawable *) drawing_area->window, obj ); */
    }

    return FALSE; /* pass on the event */
}


static gint expose_event(GtkWidget *widget, GdkEventExpose *event)
{
    /* Redraw clobbered area of backing pixmap area */
    gdk_draw_pixmap(widget->window,
                    widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
                    backing_store,
                    event->area.x, event->area.y,
                    event->area.x, event->area.y,
                    event->area.width, event->area.height);

    return FALSE;
}


static gint configure_event(GtkWidget *widget, GdkEventConfigure *event)
{
    if ( backing_store )
        gdk_pixmap_unref(backing_store);

    backing_store = gdk_pixmap_new(widget->window,
                                   widget->allocation.width,
                                   widget->allocation.height, -1);

    /* Blank out the rectangle */
    gdk_draw_rectangle( backing_store,
                        widget->style->bg_gc[GTK_WIDGET_STATE(widget)],
                        TRUE, 0, 0,
                        widget->allocation.width,
                        widget->allocation.height);

    draw_object( (GdkDrawable *) backing_store, obj );

    return TRUE;
}


void setup_colors( Kybos *obj )
{
    int i;
    GdkColor *color;
    gushort   intensity;

#define NUM_COLORS      16
#define INTENSITY       0xffff
    /* Allocate three GC's for red, green, and blue */
    obj->gcs = (GdkGC **) xmalloc(sizeof(GdkGC *) * NUM_COLORS);
    color = (GdkColor *) xmalloc(sizeof(GdkColor));
    for ( i = 0; i < NUM_COLORS; i++ ) {
        *(obj->gcs + i) = gdk_gc_new(main_window->window);
        gdk_gc_copy(*(obj->gcs + i), main_window->style->fg_gc[GTK_STATE_NORMAL]);

        if ( i & 0x08 )
            intensity = INTENSITY;
        else
            intensity = INTENSITY/2;
        color->red   = (i & 0x01) ? intensity : 0;
        color->green = (i & 0x02) ? intensity : 0;
        color->blue  = (i & 0x04) ? intensity : 0;
        color->pixel =
            ((gulong) color->red << 16)  +
            ((gulong) color->green << 8) +
            (gulong) color->blue;
        gdk_color_alloc(gtk_widget_get_colormap(main_window), color);
        gdk_gc_set_foreground(*(obj->gcs+i), color);
    }

    free(color);

    return;
}


extern int parse_options(int argc, char **argv);

int main(int argc, char **argv)
{
    int i;

    gtk_init(&argc, &argv);
    progname = *argv;

    i = parse_options(argc, argv);
    if ( i == 0 ) gtk_exit(-1);
    else if ( i == 1 ) gtk_exit(0);

    obj = read_data(stream_in);
    obj->projection = projection;

    /* Create main window */
    main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_events( main_window,
                           GDK_KEY_PRESS_MASK );
    gtk_widget_realize(main_window);
    gtk_window_set_title( GTK_WINDOW(main_window), "Kybos" );
    /* Can both shrink and grow */
    gtk_window_set_policy( GTK_WINDOW(main_window), TRUE, TRUE, FALSE );
    /* Make it close on destroy signal */
    gtk_signal_connect( GTK_OBJECT(main_window), "destroy",
                        GTK_SIGNAL_FUNC(main_destroy), NULL );

    setup_colors(obj);

    /* Create drawing area inside the main window */
    drawing_area = gtk_drawing_area_new();
    gtk_signal_connect( GTK_OBJECT(drawing_area), "expose_event",
                        GTK_SIGNAL_FUNC(expose_event), NULL );
    gtk_signal_connect( GTK_OBJECT(drawing_area), "configure_event",
                        GTK_SIGNAL_FUNC(configure_event), NULL );
    gtk_signal_connect( GTK_OBJECT (main_window), "key_press_event",
                        GTK_SIGNAL_FUNC(key_event), NULL );
    gtk_container_add( GTK_CONTAINER(main_window), drawing_area );

    /* Show all widgets */
    gtk_widget_show(drawing_area);
    gtk_widget_show(main_window);

    /* Wait for events from X server */
    gtk_main();

    return 0;
}



/*
 * Local variables:
 * compile-command:  "make -k && ./kybos < kybos.data"
 * End:
 */
