#include <assert.h>


#define draw_pixel(x,y)     ( *(scr + y*scr_width + x) = col )


void line(unsigned char *scr, int scr_width, unsigned char col,
          int x, int y, int X, int Y)
{
    int tmp, deltaX, deltaY, dirX;

    assert(scr);

    if ( y > Y ) { /* all code assumes y <= Y */
        tmp = y, y = Y, Y = tmp;
        tmp = x, x = X, X = tmp;
    }

    deltaX = X - x;
    if ( deltaX < 0 ) {
        deltaX = -deltaX; /* | deltaX | */
        dirX = -1;
    } else
        dirX = 1;
    deltaY = Y - y;       /* deltaY >= 0 */


    if ( deltaX == 0 ) {          /* vertical */
        while ( deltaY-- >= 0 )
            draw_pixel(x, y++);

        return;
    }

    else if ( deltaY == 0 ) {     /* horizontal */
        while ( deltaX-- >= 0 ) {
            draw_pixel(x, y);
            x += dirX;
        }
    }

    else if ( deltaX == deltaY ) { /* diagonal */
        while ( deltaY-- >= 0 ) {
            draw_pixel(x, y++);
            x += dirX;
        }
    }

    else if ( deltaX < deltaY ) {  /* octant 1 or 2 ( deltaX != 0 ) */
        int error, errorAdd, errorReset, thresh;

        assert( deltaY >= 0 );

        /* Scaled error threshold for moving sideways one pixel */
        thresh = deltaY;
        errorReset = 2 * thresh;
        /* Scaled x-increment corresponding to y-increment of 1 */
        errorAdd = 2 * deltaX;

        /* Initial error */
        error  = 0;

        while ( deltaY-- >= 0 ) {
            draw_pixel(x, y);
            /* Step 1 pixel up Oy, accumulating error in direction of Ox */
            y++;
            error += errorAdd;
            if ( error >= thresh ) { /* threshold reached, go sideways */
                x += dirX;
                error -= errorReset; /* adjust back the error */
            }
        }
    }

    else {                         /* octant 0 or 3 ( deltaX > deltaY != 0 ) */
        int error, errorAdd, errorReset, thresh;

        assert( deltaY >= 0 );

        /* Scaled error threshold for moving up one pixel */
        thresh = deltaX;
        errorReset = 2 * thresh;
        /* Scaled y-increment corresponding to x-increment of 1 */
        errorAdd = 2 * deltaY;

        /* Initial error */
        error  = 0;

        while ( deltaX-- >= 0) {
            draw_pixel(x, y);
            /* Step 1 pixel sideways, accumulating error in direction of Oy */
            x += dirX;
            error += errorAdd;
            if ( error >= thresh ) { /* threshold reached, go up */
                y++;
                error -= errorReset;  /* adjust back the error */
            }
        }
    }


    return;
}



/*
 * Local variables:
 * compile-command: "make -k"
 * End:
 */
