#include "threads.h"
#include <sys/time.h>
#include <signal.h>


#define POINTER_WIDTH           9
#define POINTER_HEIGHT          (sizeof(pointer)/sizeof(pointer[0]))
static unsigned char *pointer[] = {
".        ",
"..       ",
".+.      ",
".++.     ",
".+++.    ",
".++++.   ",
".+++++.  ",
".++++++. ",
".+++.....",
".+.++.   ",
".. .+.   ",
".  .++.  ",
"    .+.  ",
"    .++. ",
"     .+. ",
"      .  "
};


static int can_spawn = 1;

static int x, y, button, old_button = 0;

/* Window we are moving and offset of pointer inside it */
static window *move_win= NULL;
static point delta;


static void can_spawn_timer_handler(int signum)
{
    can_spawn = 1;

    return;
}


/* This function unsets `can_spawn' and installs a timer which will reset
 * `can_spawn' after WAIT_SPAWN_INTERVAL period expires.  The idea is to
 * disallow creation or deletion of any windows for WAIT_SPAWN_INTERVAL period
 * after creating or deleting a window to block unwanted `doubling' of events
 * from mouse and/or keyboard */
static void unset_can_spawn(void)
{
    struct itimerval itval;
    struct sigaction saction, old_saction;
    sigset_t sigset;

    if ( !can_spawn )
        return;

    can_spawn = 0;
    /* Set handler for the timer signal which will restore
     * can_spawn */
    saction.sa_handler = can_spawn_timer_handler;
    sigemptyset(&sigset);
    saction.sa_mask    = sigset;
    saction.sa_flags   = SA_RESTART;
    if ( sigaction(SIGALRM, &saction, &old_saction ) ) {
        /* Could not set the handler, forget it */
        can_spawn = 1;
        return;
    }
    /* Send signal only once */
    itval.it_interval.tv_sec = 0;
    itval.it_interval.tv_usec = 0;
    /* Timer interval value */
    itval.it_value.tv_sec  = 0;
    itval.it_value.tv_usec = WAIT_SPAWN_INTERVAL;
    /* Set the timer */
    if ( setitimer( ITIMER_REAL, &itval, NULL ) ) {
        /* Could not set the timer, forget it */
        can_spawn = 1;
        /* Restore original signal handling */
        sigaction(SIGALRM, NULL, &old_saction);
        return;
    }

    return;
}


void check_mouse(void)
{
    mouse_update();
    keyboard_update();
    if ( 1 ) {
        x = mouse_getx();
        x = mouse_getx_emulated(x);
        y = mouse_gety();
        y = mouse_gety_emulated(y);
        button = mouse_getbutton() | mouse_getbutton_emulated();;
        mouse_setposition(x, y);

        if ( button & MOUSE_LEFTBUTTON ) {
            /*
             * Left mouse button down
             */
            if ( old_button & MOUSE_LEFTBUTTON ) {
                /* We are moving a window--go on moving */
                move_win->x = x - delta.x;
                move_win->y = y - delta.y;
            } else {
                /* We are not moving a window--bring up a window and start
                 * moving it (save window's upper left corner's offset) */
                pthread_mutex_lock(&wlist_mutex);
                move_win = win_top_xy(x, y);
                win_bring_up(move_win);
                delta.x = x - move_win->x;
                delta.y = y - move_win->y;
                pthread_mutex_unlock(&wlist_mutex);
            }
        } else if ( button & MOUSE_RIGHTBUTTON ) {
            /*
             * Right mouse button down--create or delete a window
             */
            if ( can_spawn ) { /* can we already do it? */
                window *win;

                unset_can_spawn();
                /* We should not lock wlist_mutex here--otherwise thread_down() will
                 * cause dead-lock.  But this is not a problem, since only this
                 * thread (display server) can destroy windows */
                win = win_top_xy(x, y);
                if ( win )
                    /* Destroy window */
                    thread_down(win);
                else
                    /* Create new window */
                    thread_up(x, y, -1, -1, time(NULL), -1, -1, -1);
            }
        }
    }

    old_button = button;

    return;
}


/* Assumes pointer's hot-spot to be (0, 0) */
void redraw_mouse_pointer(unsigned char *scr_store)
{
    int i, j, width, height;
    int scan_start_offset_scr;
    unsigned char *ptr_scr, *ptr_img;

    assert(scr_store);

    /* Make sure we do not draw past end of scanlines and past end of backing
     * store of the screen.  Here we assume that mouse pointer's coordinates are
     * not off the screen */
    if ( x + POINTER_WIDTH > scr_width )
        /* Part of pointer image behind right edge of the screen */
        width = scr_width - x;
    else
        /* Pointer image fits in the screen (horizontally) */
        width = POINTER_WIDTH;
    scan_start_offset_scr = scr_width - width;

    if ( y + POINTER_HEIGHT > scr_height )
        /* Part of pointer image below bottom edge of the screen */
        height = scr_height - y;
    else
        /* Pointer image fits in the screen (vertically) */
        height = POINTER_HEIGHT;

    /* Initial offset in screen's backing store */
    ptr_scr = scr_store + y * scr_width + x;

    for ( i = 0; i < height; i++ ) {
        for ( j = 0, ptr_img = *(pointer+i); j < width;
              j++, ptr_img++, ptr_scr++ ) {
            switch ( *ptr_img ) {
            case '.':
                *ptr_scr = COLOR_BG;
                break;
            case '+':
                *ptr_scr = COLOR_FG;
                break;
            }
        }
        /* Go to start of next scanline in screen's backing store */
        ptr_scr += scan_start_offset_scr;
    }

    return;
}



/*
 * Local variables:
 * compile-command: "make -k"
 * End:
 */
