#ifndef __THREADS_H__
#define __THREADS_H__


#include <stdio.h>
#include <pthread.h>
#include <assert.h>
#include <time.h>
#include <stdlib.h>
#include <vga.h>
#include <vgakeyboard.h>
#include <vgamouse.h>


#define DEFAULT_WIN_WIDTH               200
#define DEFAULT_WIN_HEIGHT              150
#define DEFAULT_STARTUP_THREADS_COUNT   10

#define COLOR_BG                        0
#define COLOR_FG                        15
#define COLOR_BORDER_TL                 7
#define COLOR_BORDER_BR                 8
#define COLOR_FACE_DOT_SMALL            7
#define COLOR_FACE_DOT_BIG              7
#define COLOR_HOURS_HAND                7
#define COLOR_MINUTES_HAND              7
#define COLOR_SECONDS_HAND              6

#define WIN_PLACEMENT_TOP_PAD           (scr_height/10)
#define WIN_PLACEMENT_LEFT_PAD          (scr_width/10)
#define WIN_PLACEMENT_BOTTOM_PAD        (scr_height/10)
#define WIN_PLACEMENT_RIGHT_PAD         (scr_width/10)
#define WIN_PLACEMENT_WIDTH_MIN         (scr_width/5)
#define WIN_PLACEMENT_WIDTH_MAX         (scr_width/2)
#define WIN_PLACEMENT_HEIGHT_MIN        (scr_height/5)
#define WIN_PLACEMENT_HEIGHT_MAX        (scr_height/2)

/* How many microseconds to wait before allowing to create new windows or delete
 * existing ones.  Needed to prevent piled-up messages (two clicks when user
 * just clicked once) */
#define WAIT_SPAWN_INTERVAL             300000

/* Parameters for emulation of mouse with keyboard */
#define DEFAULT_DELTA_INIT              1
#define DEFAULT_DELTA_MAX               50

typedef struct _point {
    int x;
    int y;
} point;


typedef struct _window {
    /* Window geometry */
    int   x;
    int   y;
    int   width;
    int   height;

    /* Thread which is responsible for updating this window */
    pthread_t tid;

    /* Links to next and previous windows in stacking order */
    struct _window *next;
    struct _window *prev;

    /* Clock's startup time */
    time_t startup_time;
    /* Seconds hand's resolution */
    int    resolution;
    /* Secons hand's tick delay */
    struct timespec delay;
    /* Clock's aspect ratio mode */
    int    keep_ar;

    /* Backing store and its mutex */
    pthread_mutex_t mutex;
    unsigned char *bstore;
} window;


extern int     scr_width;
extern int     scr_height;
extern int     color_depth;
extern int     resolutions[];
extern int     resolutions_count;
extern char   *progname;

extern pthread_mutex_t wlist_mutex;
extern window *wlist_head;
extern window *wlist_tail;

extern int not_done_yet;

extern int   delta_init, delta_max;


extern int   thread_up(int x, int y, int width, int height,
                       time_t startup_time, int res_idx, long int delay,
                       int keep_ar);
extern int   thread_down(window *win);
extern void *handle_thread_cancel(void *arg);
extern void *run_clock(void *arg);
extern void  line(unsigned char *scr, int scr_width, unsigned char col,
                  int x, int y, int X, int Y);
extern int   display_server(void);
extern window *win_top_xy(int x, int y);
extern void  win_unlink(window *win);
extern void  win_bring_up(window *win);
extern void  check_mouse(void);
extern void  redraw_mouse_pointer(unsigned char *scr_store);

extern int   mouse_getbutton_emulated(void);
extern int   mouse_getx_emulated(int x);
extern int   mouse_gety_emulated(int y);


#endif /* __THREADS_H__ */
