#ifndef __WIRED_H__
#define __WIRED_H__
#include <stdio.h> /* FILE */
#include <gdk/gdk.h>



typedef struct _Point {
    double x;
    double y;
    double z;
} Point;

typedef struct _Triangle {
    int p1;                 /* these are indexes... */
    int p2;                 /* ...into array of... */
    int p3;                 /* ...points */
    int visible;
    struct _Triangle *next;
} Triangle;

typedef struct _Edge {
    int p1;                 /* these are indexes into... */
    int p2;                 /* ...array of points */
    Triangle *t1;           /* these are pointers to triangles... */
    Triangle *t2;           /* ...which contain this edge */
    struct _Edge *next;
} Edge;

typedef struct _Wired {
    int       projection;   /* 0 - parallel; 1 - perspective */
    Point     viewpoint;
    double    screen;       /* z-coordinate of the screen in view coordinates */
    int       num_vertices;
    Point    *vertices;     /* array of num_vertices vertices */
    Triangle *triangles;    /* triangulated faces */
    Edge     *edges;        /* edges of the object */
    GdkGC    *gc_solid;
    GdkGC    *gc_dashed;
    int       incr_x;       /* these are increments... */
    int       incr_y;       /* ...for movement of object along... */
    int       incr_z;       /* ...all the world axes... */
    int       incr_s;       /* ...and movement of screen along z view axis */
} Wired;


extern char  *progname;
extern void  *xmalloc(int size);
extern Wired *read_data(FILE *stream);
extern void   draw_object(GdkDrawable *drawable, Wired *obj);



#endif /* __WIRED_H__ */
